/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.circle;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.fx.StringFx;
import re.jpayet.mentdb.core.sequence.SequenceManager;
import re.jpayet.mentdb.tools.Misc;

public class CircleManager {

	//Get ids
	@SuppressWarnings("unchecked")
	public static JSONArray getIds(String relationId) throws Exception {
		
		//Initialization
		JSONArray result = new JSONArray();
		result.add(relationId);
		JSONObject test = new JSONObject();
	
		if (relationId.length()==0) {
	
			throw new Exception("Sorry, the relation id is required.");
	
		}
		
		//Try to get the relation
		JSONObject recRelation = Record.getNode(relationId);
	
		if (recRelation==null) {
	
			throw new Exception("Sorry, the relation "+relationId+" does not exist.");
	
		}
		
		String circle_id = (String) recRelation.get("c");
		if (circle_id==null) return result;
		else {
			
			result.remove(0);
			
			JSONObject c = Record.getNode(circle_id);
			
			//Save for all into the circle
			for(Object o : c.keySet()) {
				
				//get the key
				String k = (String) o;
				String r1 = Misc.atom(k, 2, " ");
				String r2 = Misc.atom(k, 3, " ");
				
				if (!test.containsKey(r1)) {
					result.add(r1);
					test.put(r1, null);
				}
				if (!test.containsKey(r2)) {
					result.add(r2);
					test.put(r2, null);
				}
				
			}
			
			return result;
			
		}
		
	}

	//Show all circles
	public static JSONObject showCircle(String user, String relationId) throws Exception {
	
		if (relationId.length()==0) {
	
			throw new Exception("Sorry, the relation id is required.");
	
		}
		
		//Try to get the relation
		JSONObject recRelation = Record.getNode(relationId);
	
		if (recRelation==null) {
	
			throw new Exception("Sorry, the relation "+relationId+" does not exist.");
	
		}
		
		String circle_id = (String) recRelation.get("c");
		if (circle_id==null) return new JSONObject();
		else {
			
			return Record.getNode(circle_id);
			
		}
		
	}

	//Add circle
	@SuppressWarnings("unchecked")
	public static void setCurrentCircle(String user, String relationId, String currentCircle) throws Exception {
		
		JSONObject recRelation = Record.getNode(relationId);
		recRelation.put("c", currentCircle);
		Record.update(user, relationId, recRelation.toJSONString());
		
	}

	//Add circle
	@SuppressWarnings("unchecked")
	public static String mergeCircle(String user, String relationId1, String positions1, String relationId2, String positions2) throws Exception {

		positions1 = StringFx.trim(positions1);
		positions2 = StringFx.trim(positions2);
		
		if (relationId1.length()==0) {
	
			throw new Exception("Sorry, the relation id is required.");
	
		}
	
		if (relationId2.length()==0) {
	
			throw new Exception("Sorry, the relation to add is required.");
	
		}
		
		//Try to get the relation 1
		JSONObject recRelation1 = Record.getNode(relationId1);
	
		if (recRelation1==null) {
	
			throw new Exception("Sorry, the relation "+relationId1+" does not exist.");
	
		}
		
		//Try to get the relation 2
		JSONObject recRelation2 = Record.getNode(relationId2);
	
		if (recRelation2==null) {
	
			throw new Exception("Sorry, the relation "+relationId2+" does not exist.");
	
		}
		
		//Parse all positions in the relation 2
		int nbPosition = Misc.size(positions2, " ");
	
		if (nbPosition!=RelationManager.showThoughtNodes(user, relationId2).size()) {
	
			throw new Exception("Sorry, the number of position is not equals to the number of thought/relation into the relation "+relationId2+".");
	
		}
		
		//Parse all positions in the relation 1
		nbPosition = Misc.size(positions1, " ");
	
		if (nbPosition!=RelationManager.showThoughtNodes(user, relationId1).size()) {
	
			throw new Exception("Sorry, the number of position is not equals to the number of thought/relation into the relation "+relationId1+".");
	
		}
		
		//Get the circle 2
		String circle_id_2 = ((String) recRelation2.get("c"));
		JSONObject currentCircle2 = null;
		if (circle_id_2==null) currentCircle2 = new JSONObject();
		else {
			currentCircle2 = Record.getNode(circle_id_2);
		}
		
		//Lang
		String lang1 = (String) recRelation1.get("l");
		String lang2 = (String) recRelation2.get("l");
		
		//Get the circle 1
		String circle_id_1 = ((String) recRelation1.get("c"));
		JSONObject currentCircle1 = null;
		if (circle_id_1==null) currentCircle1 = new JSONObject();
		else {
			currentCircle1 = Record.getNode(circle_id_1);
		}
		
		//Add circle 2 into the circle 1
		for(Object o2 : currentCircle2.keySet()) {
			
			//get the key
			String k = (String) o2;
			String val = (String) currentCircle2.get(k);
			
			if (!currentCircle1.containsKey(k)) {
				
				currentCircle1.put(k, val);
				
			}
			
		}
		
		//Add the link into the circle
		currentCircle1.put(lang1+" "+relationId2+" "+relationId1, positions2);
		//Inverse the position
		currentCircle1.put(lang2+" "+relationId1+" "+relationId2, positions1);
		
		if (circle_id_1 == null) {
			
			circle_id_1 = "CI["+SequenceManager.incr(user, "circle")+"]";
			Record.add(user, circle_id_1, currentCircle1.toJSONString());
			
		} else {
			
			Record.update(user, circle_id_1, currentCircle1.toJSONString());
			
		}
		
		if (circle_id_2 != null && !circle_id_2.equals(circle_id_1)) {
			
			Record.remove(user, circle_id_2);
			
		}
		
		//Save for all into the circle1
		for(Object o1 : currentCircle1.keySet()) {
			
			//get the key
			String k = (String) o1;
			String r1 = Misc.atom(k, 2, " ");
			String r2 = Misc.atom(k, 3, " ");
			
			setCurrentCircle(user, r1, circle_id_1);
			setCurrentCircle(user, r2, circle_id_1);
			
		}
		
		//Save for all into the circle2
		for(Object o2 : currentCircle2.keySet()) {
			
			//get the key
			String k = (String) o2;
			String r1 = Misc.atom(k, 2, " ");
			String r2 = Misc.atom(k, 3, " ");
			
			setCurrentCircle(user, r1, circle_id_1);
			setCurrentCircle(user, r2, circle_id_1);
			
		}
		
		return circle_id_1;
		
	}

	//Delete circle
	@SuppressWarnings("unchecked")
	public static void deleteCircle(String user, String relationIdToDelete) throws Exception {
	
		if (relationIdToDelete.length()==0) {
	
			throw new Exception("Sorry, the relation to delete is required.");
	
		}
		
		//Try to get the relation 1
		JSONObject recRelation = Record.getNode(relationIdToDelete);
	
		if (recRelation==null) {
	
			throw new Exception("Sorry, the relation "+relationIdToDelete+" does not exist.");
	
		}
		
		//Get the current circle
		String circle_id = ((String) recRelation.get("c"));
		JSONObject currentCircle = null;
		if (circle_id==null) currentCircle = new JSONObject();
		else {
			currentCircle = Record.getNode(circle_id);
		}
		
		//Initialize the deleted keys
		JSONArray deletedKeys = new JSONArray();
		JSONObject deletedRelations = new JSONObject();
		
		//Delete all occurrences
		for(Object o1 : currentCircle.keySet()) {
			
			//get the current key
			String k = (String) o1;
			
			if (k.indexOf(relationIdToDelete)>-1) {
				
				deletedKeys.add(k);
	
				deletedRelations.put(Misc.atom(k, 2, " "), false);
				deletedRelations.put(Misc.atom(k, 3, " "), true);
				
			}
			
		}
		for(int i=0;i<deletedKeys.size();i++) {
			currentCircle.remove((String) deletedKeys.get(i));
		}
		
		if (circle_id != null) {
			
			if (currentCircle.size()==0) Record.remove(user, circle_id);
			else Record.update(user, circle_id, currentCircle.toJSONString());
			
		}
		
		//Clear the circle for all deleted keys
		for(Object d : deletedRelations.keySet()) {
			
			//get the key
			String r = (String) d;
			
			setCurrentCircle(user, r, null);
			
		}
		
		//Save for all into the circle
		for(Object o1 : currentCircle.keySet()) {
			
			//get the key
			String k = (String) o1;
			String r1 = Misc.atom(k, 2, " ");
			String r2 = Misc.atom(k, 3, " ");
			
			setCurrentCircle(user, r1, circle_id);
			setCurrentCircle(user, r2, circle_id);
			
		}
		
	}

	//Exist circle
	public static String existCircle(String user, String relationToFind) throws Exception {
	
		if (relationToFind.length()==0) {
	
			throw new Exception("Sorry, the relation to find is required.");
	
		}
		
		//Try to get the relation
		JSONObject recRelation = Record.getNode(relationToFind);
	
		if (recRelation==null) {
	
			throw new Exception("Sorry, the relation "+relationToFind+" does not exist.");
	
		}
		
		String currentCircle = ((String) recRelation.get("c"));
		
		if (currentCircle==null) return "0";
		return "1";
		
	}
	
	

}
