/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.entity.language;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;

import re.jpayet.mentdb.core.db.Database;
import re.jpayet.mentdb.core.db.Record;
import re.jpayet.mentdb.core.db.file.node.NodeManager;
import re.jpayet.mentdb.core.entity.concentration.ConcentrationManager;
import re.jpayet.mentdb.core.entity.link.LinkManager;
import re.jpayet.mentdb.core.entity.relation.RelationManager;
import re.jpayet.mentdb.core.entity.thought.ThoughtManager;
import re.jpayet.mentdb.core.entity.word.WordManager;
import re.jpayet.mentdb.core.user.GroupManager;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

public class LanguageManager {
	
	public static JSONArray firstLang = new JSONArray();
	
	//Stimulate a language
	@SuppressWarnings("unchecked")
	public static void stimulate(String cuser, String lang) throws Exception {
		
		//Get the language node
		JSONObject rec = Record.getNode("L["+lang+"]");
		if (rec==null) {
			throw new Exception("Sorry, the language id "+lang+" does not exist."); 
		}
		
		RelationManager.stimulate(cuser, ""+rec.get("rl"), true);
		
		firstLang.remove(lang);
		
		firstLang.add(0, lang);
		
	}
	
	//Check if a language already exist
	public static void exceptionIfNotExist(String lang) throws Exception {
		
		if (Database.isNotInit && !firstLang.contains(lang)) {
			
			throw new Exception("Sorry, the language "+lang+" does not exist."); 
		
		}
		
	}
	
	//Show first language
	@SuppressWarnings("unchecked")
	public static String first(String cuser) throws Exception {
		
		if (firstLang.size()==0) {
		
			JSONArray languages = RelationManager.showTabLinks(cuser, "["+LinkManager.get_value("@[r_fr_?_est_une_langue]")+" 2/3]", "L", ConcentrationManager.getConcentrationDepth("C[relation]"));
			
			//Parse all language relation tab links
			for(int i=0;i<languages.size();i++) {
				
				if (i==ConcentrationManager.getConcentrationDepth("C[language]")) break; 
				
				//Get the current language relation
				String languageRelationTabLink = languages.get(i)+"";
				
				String relationId = Misc.atom(languageRelationTabLink, 1, " ").substring(1);
				
				JSONArray words = RelationManager.showWords(cuser, relationId);
				
				String lang = ((JSONObject) ((JSONArray) words.get(0)).get(0)).get("word")+"";
				lang = lang.substring(2, lang.length()-1);
				
				firstLang.add(lang);
				
			}
			
		}
		
		return firstLang.get(0)+"";
		
	}
	
	//Show all languages
	public static JSONArray showAll() throws Exception {
		
		return firstLang;
		
	}
	
	//Create a new language
	@SuppressWarnings("unchecked")
	public static void create(String cuser, String language) throws Exception {
		
		if (!cuser.equals("mentdb") && !GroupManager.isGrantedUser(cuser, "system", cuser)) {
			
			throw new Exception("Sorry, only user in 'system' group can create a new language."); 
			
		}
		
		//Check parameters
		if (Misc.lrtrim(language).equals("")) {
			throw new Exception("Sorry, the language id is required char(2)."); 
		}
		
		//Check parameters
		if (language.length()!=2) {
			throw new Exception("Sorry, the language id must be 2 characters."); 
		}
		
		//Get the language node
		JSONObject rec = Record.getNode("L["+language+"]");
		if (rec!=null) {
			throw new Exception("Sorry, the language id "+language+" already exist."); 
		}
		
		//Add the language thought
		String languageThought = "";
		
		if (!WordManager.exist(cuser, language, Start.FIRST_LANGUAGE)) {
			languageThought = WordManager.create(cuser, language, "", Start.FIRST_LANGUAGE);
		} else {
			languageThought = ThoughtManager.create(cuser, language, "", Start.FIRST_LANGUAGE);
		}
		
		//Add the language
		NodeManager.oinsert(cuser, languageThought, "/", "language", language, "STR");
		
		//Create the language relation
		String relationId = RelationManager.createSubWithPosition(cuser, LinkManager.get_value("@[r_fr_?_est_une_langue]"), languageThought+" "+LinkManager.get_value("@[th_fr_est]")+" "+LinkManager.get_value("@[r_fr_une_langue]"), "1 2 3", Start.FIRST_LANGUAGE, "L");
		
		JSONObject languageDirectAccess = new JSONObject();
		languageDirectAccess.put("th", languageThought);
		languageDirectAccess.put("rl", relationId);
		Record.add(cuser, "L["+language+"]", languageDirectAccess.toJSONString());
		
		firstLang.add(0, language);
		
	}

}
