/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.session;

import java.util.HashMap;
import java.util.Vector;

import org.eclipse.jetty.websocket.api.Session;
import org.eclipse.jetty.websocket.api.WebSocketAdapter;
import org.eclipse.jetty.websocket.api.annotations.WebSocket;
import org.json.simple.JSONArray;
import org.json.simple.parser.JSONParser;

import re.jpayet.mentdb.MentDBConnector;
import re.jpayet.mentdb.core.basic.BasicResult;
import re.jpayet.mentdb.core.command.MQLValue;
import re.jpayet.mentdb.core.log.Log;
import re.jpayet.mentdb.core.stat.Statistic;
import re.jpayet.mentdb.core.user.UserManager;
import re.jpayet.mentdb.server.Start;
import re.jpayet.mentdb.tools.Misc;

//Web socket management
@WebSocket
public class WebSocketThread extends WebSocketAdapter {
	
	//Static
	public static HashMap<Long, WebSocketThread> allSessions = new HashMap<Long, WebSocketThread>();
	public static HashMap<String, Vector<String>> target = new HashMap<String, Vector<String>>();
	
	//Object
	public Session session ;
	public long idConnection = 0;
	public static long nbConnection = 0;
	public static long nbOpenConnection = 0;
	public static long exceededSessions = 0;
	public long cmdId = 0;
	public String user = "", password = "";
	public MentDBConnector mentdb = null;
	public boolean isValidSession = false;
	boolean alreadyOpenSession = false;
	public boolean mqlQueryMode = true;
	public boolean isGrantedApiMql = false;
	public boolean isGrantedApiAi = false;

	//A user cannot open 2 sessions in the same time
	public static synchronized boolean openCloseSession(String user, boolean openSession, boolean isValidSession, WebSocketThread session) {
		
		//Initialization
		boolean alreadyOpen = false;
		
		if (openSession) {
			
			//If open the session
			if (re.jpayet.mentdb.core.session.Session.allSessions.containsKey(user)) {
				
				alreadyOpen = true;
				
			} else {
				
				//Add the user in the list
				re.jpayet.mentdb.core.session.Session.allSessions.put(user, session);
				
			}
			
		} else {
			
			//If close the session
			//Close the session if the session is valid
			if (isValidSession) {
				
				re.jpayet.mentdb.core.session.Session.allSessions.remove(user);
			
			}
			
		}
		
		return alreadyOpen;
		
	}
	
	@Override
	public void onWebSocketConnect(Session sess) {
		
		//Increment the number of thread
		nbOpenConnection++;
		
		nbConnection++;
		Statistic.incrementNbSession();
		this.idConnection = nbConnection;
		this.session = sess;
		
		allSessions.put(this.idConnection, this);
		
		//Check the max session
		try {
			
			if (nbOpenConnection>Start.MAX_SESSION) {
				
				//Increment the number of exceeded sessions
				exceededSessions++;

				//Trace in log file
				Log.write("Web Socket: Maximum number of sessions exceeded.");
				
				this.session.close(0, "Sorry, maximum number of sessions exceeded.");
				
			}
		} catch (NumberFormatException e1) {
		} catch (Exception e1) {
		}
		
	}

	@Override
    public synchronized void onWebSocketText(String inputText) {
		
		//Only 'LF' for the end of lines and LRTRIM
		inputText = Misc.lrtrim(inputText.replace("\r\n", "\n").replace("\r", "\n"));
		
		//Try to get streams
		try {

			//New command
			cmdId++;
			
			//Initialize the command line object
			Vector<Vector<MQLValue>> inputVector = null;
			
			if (cmdId==1) {
				
				inputVector = Misc.splitCommand(inputText);
				
				try {
					
					if (Misc.isCmdSession("#login", inputVector.get(0), session, 3)) {

						//Get the user and the password
						user = inputVector.get(0).get(1).value;
						password = inputVector.get(0).get(2).value;
						
					} else {

						throw new Exception("Sorry, Invalid first command.");
						
					}
					
					try {
						
						if (UserManager.connect(user, password, "WS")) {
							
							mentdb = re.jpayet.mentdb.core.session.Session.allConnections.get(user);
							
							//Add the new session
							isValidSession = true;
							
							openCloseSession(user, true, isValidSession, this);
							
							//Add default target
							Vector<String> targetUserList = new Vector<String>();
							targetUserList.add("ai");
							target.put(user, targetUserList);
							
							String result = mentdb.execute("group is granted user \""+user.replace("\"", "\\\"")+"\" \"api-mql\";");
							if (result.equals("1")) {
								
								isGrantedApiMql = true;
								
							}
							
							result = mentdb.execute("group is granted user \""+user.replace("\"", "\\\"")+"\" \"api-ai\";");
							if (result.equals("1")) {
								
								isGrantedApiAi = true;
								
							}
							
							//Granted connection
							result = mentdb.execute("user show;");
							JSONParser jsonParser = new JSONParser();
							JSONArray a = (JSONArray) jsonParser.parse(result);
							BasicResult br = new BasicResult(user, "ai", 1, 
									"----------------------------------------------\n"+
									"-  Welcome to MentDB !        v_"+Start.version+"  -\n"+
									"-                                            -\n"+
									"-  The Mentalese Database Engine             -\n"+
									"-  By Jimmitry Payet, Programmer Analyst     -\n"+
									"-  contact@jpayet.re, http://www.mentdb.org  -\n"+
									"-  © "+Start.copyright+", all rights reserved.         -\n"+
									"-                                            -\n"+
								    Misc.rpad("-  SID: "+idConnection+", User: "+user, " ", "45")+"-\n"+
								    "----------------------------------------------", Misc.JSONArray2StringTab(a), WebSocketThread.target.get(user), UserManager.allUsersWhoTalkingWith(user), 0);
							session.getRemote().sendString(br.toString());
							re.jpayet.mentdb.core.session.Session.sendMessageToOthers(user, 2, user+" is connected.");
							
						} else {
							
							//This clause is never pass
							
						}
						
					} catch (Exception f) {
						
						//Database connection error
						BasicResult br = null;
						if ((f.getMessage()+"").startsWith("jajidfm62mr7i8dtyfr2tyrypzea8tyyoejht") || (f.getMessage()+"").startsWith("j12hki95orm35hrm62vni90tkmr33sdy")) br = new BasicResult("", "ai", 0, f.getMessage(), 0);
						else br = new BasicResult("", "ai", 0, "j12hki95orm35hrm62vni90tkmr33sdy9", 0);
						session.getRemote().sendString(br.toString());
						session.close(0, "exit");
						
					}
					
				} catch (Exception e) {
					
					//Is not a valid connection
					BasicResult br = new BasicResult("", "ai", 0, "j12hki95orm35hrm62vni90tkmr33sdy8", 0);
					session.getRemote().sendString(br.toString());
					session.close(0, "exit");
					
				}
				
			} else {
				
				//Handle the command
				if (!inputText.startsWith("#j8i9m0m5i5t3r2y1p9a6y4e5t") || inputText.indexOf(" ")>-1) {
					
					inputVector = Misc.splitCommand(inputText);
					re.jpayet.mentdb.core.session.Session.sendMessageToTargetsAndMe(user, 1, inputText, inputVector);
					
				} else {
					
					if (inputText.startsWith("#j8i9m0m5i5t3r2y1p9a6y4e5t%")) {
						
						try {
							
							re.jpayet.mentdb.core.session.Session.allSessions.get(user).mqlQueryMode=false;
							
						} catch (Exception e) {}
						
						try {
							re.jpayet.mentdb.core.session.Session.sendMessageToMe(user, 4, "Refresh.");
						} catch (Exception e) {}
						
					} else if (inputText.startsWith("#j8i9m0m5i5t3r2y1p9a6y4e5t$")) {
						
						try {
							
							re.jpayet.mentdb.core.session.Session.allSessions.get(user).mqlQueryMode=true;
							
						} catch (Exception e) {}
						
						try {
							re.jpayet.mentdb.core.session.Session.sendMessageToMe(user, 4, "Refresh.");
						} catch (Exception e) {}
						
						
					} else {
						
						//Update target
						if (target.get(user).contains(inputText.substring(27))) {
							
							target.get(user).remove(inputText.substring(27));
							
						} else {
							target.get(user).add(inputText.substring(27));
						}
						
						re.jpayet.mentdb.core.session.Session.sendTargetToMeAndAnother(user, inputText.substring(27), 4, "Target <"+inputText.substring(27)+"> updated.");
						
					}
					
				}
				
				Statistic.incrementNbCommand();
								
			}

		} catch (Exception e) {
			
			try {
				re.jpayet.mentdb.core.session.Session.mentdbSendMessageToMe(user, 0, ""+e.getMessage(), 0);
			} catch (Exception f) {
				
			}

		}
        
    }
	
	@Override
	public void onWebSocketClose(int statusCode, String reason) {
		
		//Decrement the number of thread
		nbOpenConnection--;
		
		//Clear target list
		target.remove(user);
		
		//Remove the session
		try {
			
			if (isValidSession && re.jpayet.mentdb.core.session.Session.allConnections.containsKey(user)) {
				try {re.jpayet.mentdb.core.session.Session.allConnections.get(user).close();} catch (Exception e) {};
				try {re.jpayet.mentdb.core.session.Session.allConnections.remove(user);} catch (Exception e) {};
			}
			
		} catch (Exception e) {};
		allSessions.remove(this.idConnection);
		
		//Check if the session is already open
		if (!alreadyOpenSession) {
			
			openCloseSession(user, false, isValidSession, this);
			
			try {
				re.jpayet.mentdb.core.session.Session.sendMessageToOthers(user, 3, user+" is disconnected.");
			} catch (Exception e) {}
		
		}
        
	}

}
