/**
 * Project: MentDB
 * Description: Mentalese Database Engine
 * Website: http://www.mentdb.org
 * Twitter: https://twitter.com/mentalese_db
 * LinkedIn: https://www.linkedin.com/company/mentdb
 * Facebook: https://www.facebook.com/mentdb
 * Author: Jimmitry Payet
 * Website: http://www.jpayet.re
 * Mail: contact@jpayet.re
 * Locality: Reunion Island (French)
 */

package re.jpayet.mentdb.core.statement;

import java.util.Vector;

import re.jpayet.mentdb.core.command.Command;
import re.jpayet.mentdb.core.command.MQLValue;
import re.jpayet.mentdb.core.session.SessionThread;
import re.jpayet.mentdb.core.session.env.EnvManager;
import re.jpayet.mentdb.tools.Misc;

//The statement class
public class Statement {
	
	//EXCEPTION statement
	public static String eval(SessionThread session, String mql, EnvManager env) throws Exception {

		return Command.executeAllCommands(false, session, Misc.splitCommand(mql), env);
		
	}
	
	//EXCEPTION statement
	public static String exception_mql(SessionThread session, String id, String message, EnvManager env) throws Exception {

		String id_str = Command.executeAllCommands(false, session, Misc.splitCommand(id), env);
		String message_str = Command.executeAllCommands(false, session, Misc.splitCommand(message), env);
		
		//Set the error message
		env.set("[ERROR]", id_str+": "+message_str);
		
		//Generate an exception
		throw new Exception(id_str+": "+message_str);
		
	}
	
	//TRY statement
	public static String try_mql(SessionThread session, String mainCommand, String commandIfError, String exceptionId, EnvManager env) throws Exception {
		
		//Initialization
		String result = "";
		
		//Generate an error if the variable name is not valid
		if (!EnvManager.is_valid_varname(exceptionId)) {
			throw new Exception("Sorry, the variable name "+exceptionId+" is not valid (example: [var1]).");
		}

		//Try
		try {
			
			//Try to execute the first expression
			result=Command.executeAllCommands(false, session, Misc.splitCommand(mainCommand), env);
			
			//Clear the error message if exist
			if (env.exist(exceptionId)) {
				env.remove(exceptionId);
			}
			
		} catch (Exception e) {
			
			//Set the error message in the environment variable
			env.set(exceptionId, ""+e.getMessage());
			
			//An error appears then execute the next expression
			result=Command.executeAllCommands(false, session, Misc.splitCommand(commandIfError), env);
			
		}

		//Return the result of the function
		return result;
		
	}
	
	//REPEAT statement
	public static String repeat_mql(SessionThread session, String condition, String action, EnvManager env) throws Exception {
		
		//Initialization
		String result = "", conditionResult = null;
		
		//While loop
		do {

			//Execute the action
			result = Command.executeAllCommands(false, session, Misc.splitCommand(action), env);

			//Retest the condition
			conditionResult = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);
			
		} while (conditionResult!=null && conditionResult.equals("1"));

		//Return the result of the function
		return result;
		
	}
	
	//WHILE statement
	public static String while_mql(SessionThread session, String condition, String action, EnvManager env) throws Exception {
		
		//Initialization
		String result = "", conditionResult = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);
		
		//While loop
		while (conditionResult!=null && conditionResult.equals("1")) {

			//Execute the action
			result=Command.executeAllCommands(false, session, Misc.splitCommand(action), env);

			//Retest the condition
			conditionResult = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);
		}

		//Return the result of the function
		return result;
		
	}
	
	//FOR statement
	public static String for_mql(SessionThread session, String init, String condition, String increment, String action, EnvManager env) throws Exception {
		
		//Execute the initialization
		Command.executeAllCommands(false, session, Misc.splitCommand(init), env);
		
		//Initialization
		String result = "", conditionResult = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);

		//For loop
		while (conditionResult!=null && conditionResult.equals("1")) {

			//Execute the action
			result = Command.executeAllCommands(false, session, Misc.splitCommand(action), env);
			
			//Increment
			Command.executeAllCommands(false, session, Misc.splitCommand(increment), env);
			
			//Reset the condition result
			conditionResult = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);

		}
		
		//Return the result of the function
		return result;
		
	}
	
	//IF statement
	public static String if_mql(SessionThread session, String condition, String trueAction, EnvManager env) throws Exception {
		
		//Execute the condition
		String condition_result = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);
		
		if (condition_result!=null && condition_result.equals("1")) {
			return Command.executeAllCommands(false, session, Misc.splitCommand(trueAction), env);
		} else {
			return "";
		}
		
	}
	
	//IF statement
	public static String if_mql(SessionThread session, String condition, String trueAction, String falseAction, EnvManager env) throws Exception {
		
		//Execute the condition
		String condition_result = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);
		
		if (condition_result!=null && condition_result.equals("1")) {
			return Command.executeAllCommands(false, session, Misc.splitCommand(trueAction), env);
		} else {
			return Command.executeAllCommands(false, session, Misc.splitCommand(falseAction), env);
		}
		
	}
	
	//IF statement (force)
	public static String if_force_mql(SessionThread session, String condition, String trueAction, String falseAction, EnvManager env) throws Exception {
		
		//Execute the condition
		String condition_result = Command.executeAllCommands(false, session, Misc.splitCommand(condition), env);

		String rt = Command.executeAllCommands(false, session, Misc.splitCommand(trueAction), env);
		String rf = Command.executeAllCommands(false, session, Misc.splitCommand(falseAction), env);
		
		if (condition_result!=null && condition_result.equals("1")) {
			return rt;
		} else {
			return rf;
		}
		
	}
	
	//Case statement
	public static String case_statement(SessionThread session, Vector<MQLValue> parameters, EnvManager env) throws Exception {

		//Initialization
		String result = null, resultCondition = null;;
		boolean alreadyExecuted = false;
		
		//If the number of parameter = 1
		if (parameters.size()==1) {
			
			//Execute the else case
			return Command.executeAllCommands(false, session, Misc.splitCommand(parameters.get(0).value), env);
			
		} else {
			
			//Get the number of pairs
			int nbParamDiviseBy2 = Math.abs(parameters.size()/2);
			
			//Parse all pairs
			for(int i=0;i<nbParamDiviseBy2;i++) {
				
				//Evaluate the condition
				resultCondition = Command.executeAllCommands(false, session, Misc.splitCommand(parameters.get(i*2).value), env);
				
				//Execute if test case is true
				if (resultCondition!=null && resultCondition.equals("1")) {
					
					result = Command.executeAllCommands(false, session, Misc.splitCommand(parameters.get(i*2+1).value), env);
					alreadyExecuted = true;
					break;
					
				}
				
			}
			
			//The last parameter
			if (!alreadyExecuted && (nbParamDiviseBy2*2)!=parameters.size()) {

				result = Command.executeAllCommands(false, session, Misc.splitCommand(parameters.get(parameters.size()-1).value), env);
				
			}
			
		}

		//Return the result of the function
		return result;

	}

}
